#tree_details {
    padding:0;
    padding-top:6px;
    top: 29px;
}


#id_autocomplete {
    margin-top: -2px;
}
#id_autocomplete:active {
    box-shadow: 0 0 10px #ffffff
}
#id_autocomplete:focus {
    box-shadow: 0 0 10px #ffffff
}

ul.ui-autocomplete{
    background: #ffffff;
    border:solid 1px #D4D6D8;
    width: 135px;
    max-height: 400px;
    overflow: hidden;
    overflow-y: scroll;
}

.ui-menu-item a.ui-state-focus{
    background: #D4D6D8;
}

.ui-menu-item a{
    background:#ffffff;
    font-size: 10px;
    width: 135px;
}

.autocompletesearch label{
    top:3px;
}
